#!/usr/bin/env python2
# -*- coding: UTF-8 -*-

import os
import re
import traceback
from snack import *
from _gmutils.config import *
from _gmutils.option import Option
from _gmutils.logger import Logger
from graphicsmode import XRandr, gmMain

log = Logger()

class MainForm(object):
    screensItems = {'all': 'Все', 'one': '1', 'two': '2'}
    setMode = Option('setMode', 'list', 'main', 'Изменять настройки', 'Если выключено, используется автоматическая настройка.',
            'False', ScreenConfiguration.boolItems)
    screens = Option('screens', 'list', 'main', 'Используемые экраны', 'Количество используемых экранов',
            'all', screensItems)
    cashScreen = Option('screen', 'screen', str(), 'Интерфейс основного экрана', 'Настройка основного экрана', 'Не сконфигурирован')
    secondScreen = Option('screen', 'screen', str(), 'Экран 2', 'Настройка второго экрана', 'Не сконфигурирован')

    def __init__(self):
        # создание экрана
        self.screen = SnackScreen()
        self.screen.setColor('ROOT', 'white', 'brightmagenta')
        self.screen.setColor('HELPLINE', 'white', 'brightmagenta')

        self.config = Config()
        self.xRandr = XRandr()

        # проверяем подлючены ли по SSH!
        self.ssh = True if 'SSH_TTY' in os.environ else False

        if self.ssh:
            self.bb = ButtonBar(self.screen, (('Применить (A)', 'apply'), ("Сохранить (S)", "save"), ("Выход (ESC)", "exit")), compact = 1)
        else:
            self.bb = ButtonBar(self.screen, (("Сохранить (S)", "save"), ("Выход (ESC)", "exit")), compact = 1)

        self.screenConfigurations = []
        while len(self.screenConfigurations) < 2:
            self.screenConfigurations.append(ScreenConfiguration(notAssigned))

        tmpConfigurations = self.config.getScreenConfigurations()
        for configuration in tmpConfigurations:
            if configuration.primary.value == 'True':
                self.screenConfigurations[0] = configuration
            else:
                self.screenConfigurations[1] = configuration
                
        self.screenConfigurations[0].primary.value = 'True'
        self.screenConfigurations[1].primary.value = 'False'

        # текст бокс для описаний опции
        self.textBox = Textbox(50, 4, '', wrap = 1)

        # список с настройками
        self.optionsList = Listbox(10, width = 50, returnExit = 1)
        self.optionsList.setCallback(callBack, [self.optionsList, self.textBox])

    def start(self):
        try:
            log.info(u"Получаем список режимов")
            modes, interfaces = self.xRandr.getModes()
        except Exception as e:
            log.error(u"Не удалось получить список режимов: %s" % repr(e))
            self.messageBox('Не удалось получить список режимов', str(e), (("ОК", "ok"),))

        mode = self.config.getString('main', 'mode', '800x600')
        if mode not in modes:
            self.config.set('main', 'mode', '' if not modes else modes[0])

        g = GridForm(self.screen, "Настройка экранов", 1, 4)
        g.add(self.optionsList, 0, 1)
        g.add(self.textBox, 0, 2)
        g.add(self.bb, 0, 3)

        # добавление горячих клавиш
        g.addHotKey("ESC")
        g.addHotKey('s')
        g.addHotKey('a')
        current = ''

        while True:
            self.__createOptionsList(modes)

            try:
                self.optionsList.setCurrent(current)
            except Exception as e:
                pass

            self.textBox.setText(self.optionsList.current().specification)
            # отображаем форму
            result = g.run()
            current = self.optionsList.current()
            resultBtn = self.bb.buttonPressed(result)

            # обработка результатов выполнения
            if result == self.optionsList:
                if current == self.screens:
                    val = listDlg(self.screen, current.params, self.config.getOption(current), current.text)
                    # если настраиваются все мониторы - необходимо очистить конфигурацию от индивидуальных настроек
                    if val == 'all':
                        self.screenConfigurations[0] = ScreenConfiguration(notAssigned)
                        self.screenConfigurations[1] = ScreenConfiguration(notAssigned)
                        self.config.removeScreenSections()
                    elif val == 'one':
                        self.screenConfigurations[1] = ScreenConfiguration(notAssigned)
                        self.config.removeLastScreenSection()
                    self.config.setOption(current, val)
                elif current.type == 'list':
                    val = listDlg(self.screen, current.params, self.config.getOption(current), current.text)
                    log.info("Диалог ответ: " + val)
                    self.config.setOption(current, val)
                elif current.type == 'entry':
                    val = entryDlg(self.screen, self.config.getOption(current), current.text)
                    self.config.setOption(current, val)
                elif current == self.cashScreen:
                    log.info("Настройка основного экрана")
                    self.config.setScreenConfiguration(self.screenConfigurationDlg(self.cashScreen), primary=True)
                elif current == self.secondScreen:
                    log.info("Настройка второго экрана")
                    self.config.setScreenConfiguration(self.screenConfigurationDlg(self.secondScreen), primary=False)
            elif resultBtn == 'save' or result == "s":
                if self.__checkConfig("сохранения"):
                    self.__saveConfig()
            elif (resultBtn == 'apply' or result == "a") and self.ssh:
                if self.__checkConfig("применения"):
                    self.__applyConfig()                
            elif resultBtn == 'exit' or result == "ESC" :
                break

    def finish(self):
        if self.screen:
            self.screen.finish()

    def screenConfigurationDlg(self, cfgScreen):
        """
        Функция показа диалогового окна с полем ввода
        :param cfg - конфигурация экрана ScreenConfiguration
        """
        primary = cfgScreen == self.cashScreen
        cfg = self.screenConfigurations[0] if primary else self.screenConfigurations[1]
        buttons = ButtonBar(self.screen, [("Выход (ESC)", "exit")], compact=1)
        textBox = Textbox(48, 3, '', wrap=1)
        form = GridForm(self.screen, cfgScreen.specification, 20, 65)

        options = Listbox(len(cfg.getOptions(primary)) + 2, width=48, returnExit=1)
        options.setCallback(callBack, [self.optionsList, textBox])

        form.addHotKey("ESC")
        form.add(options, 1, 1)
        form.add(textBox, 1, 2)
        form.add(buttons, 1, 3)

        modesTmp, allInterfaces = self.xRandr.getModes()
        modes = self.xRandr.getInterfacesModes()
        options.setCallback(callBack, [options, textBox])
        selectedOption = ''

        textBox.setText(cfg.interfaceOption.specification)

        availInterfaces = [notAssigned] + [i for i in allInterfaces if i not in self.config.getUsedInterfaces()]
        if cfg.interfaceOption.value not in availInterfaces:
            availInterfaces.append(cfg.interfaceOption.value)
        cfg.setInterfaces(availInterfaces)

        while True:
            options.clear()
            options.append(formatListLine(cfg.interfaceOption.text, cfg.interfaceOption.value), cfg.interfaceOption)

            if cfg.interfaceOption.value != notAssigned:
                cfg.setModes(modes[cfg.interfaceOption.value])
                cfg.setSection(cfg.interfaceOption.value)
                if not cfg.modeOption.value:
                    cfg.modeOption.value = modes[cfg.interfaceOption.value][0] if modes[cfg.interfaceOption.value] else ''

                for option in cfg.getOptions(primary):
                    options.append(formatListLine(option.text, option.getValueDescription()), option)
            try:
                options.setCurrent(selectedOption)
            except Exception as e:
                pass

            result = form.run()
            selectedOption = options.current()

            if result == options:
                if selectedOption.type == 'list' or selectedOption.type == 'dict':
                    selectedOption.value = listDlg(self.screen, selectedOption.params, selectedOption.value, selectedOption.text)
                elif selectedOption.type == 'entry':
                    selectedOption.value = entryDlg(self.screen, selectedOption.value, selectedOption.text)
                elif selectedOption.type == 'listWithEntry':
                    selectedOption.value = listWithEntryDlg(self.screen, selectedOption.params, selectedOption.value, selectedOption.text)
            elif buttons.buttonPressed(result) == 'exit' or result == "ESC":
                self.screen.popWindow()
                return cfg

    def __createOptionsList(self, modes):
        '''
        Очистка и заполнение self.optionsList опциями
        '''
        mode = self.config.getString('main', 'mode', '800x600')
        modeOption = Option('mode', 'list', 'main', 'Разрешение экрана', 'Разрешение для всех мониторов', mode if mode in modes else '', modes)
        fbOption = Option('fb', 'entry', 'main', 'Размер скрина',
                'Переконфигурация экрана до указанного размера.\nПо умолчанию равно разрешению экрана', '')

        self.optionsList.clear()
        self.optionsList.append(formatListLine(self.setMode.text, ScreenConfiguration.boolItems[self.config.getOption(self.setMode)]), self.setMode)

        screensOption = self.config.getOption(self.screens)
        if self.config.getOption(self.setMode) == 'True':
            self.optionsList.append(formatListLine(self.screens.text, self.screensItems[self.config.getOption(self.screens)]), self.screens)
            if screensOption == 'one' or screensOption == 'two':
                self.optionsList.append(formatListLine(" Основной экран", self.screenConfigurations[0].getAnnotation()), self.cashScreen)
                if screensOption == 'two':
                    self.optionsList.append(formatListLine(" Второй экран", self.screenConfigurations[1].getAnnotation()), self.secondScreen)
            else:
                self.optionsList.append(formatListLine(modeOption.text, self.config.getOption(modeOption)), modeOption)
            self.optionsList.append(formatListLine(fbOption.text, self.config.getOption(fbOption)), fbOption)

    def __checkConfig(self, action):
        """
        Проверка валидности конфигурации перед сохранением или применением
        """
        error = ""
        if self.config.getOption(self.screens) != "all" and self.screenConfigurations[0].interfaceOption.value == notAssigned:
            error = "Не указан интерфейс основного экрана"
        
        if len(error) > 0:
            self.messageBox('Ошибка %s конфигурации' % action, error, (("ОК", "ok"),))
            return  False

        return True

    def __saveConfig(self):
        try:
            self.config.write()
        except Exception as e:
            log.error(u"Ошибка сохранения: %s" % str(e))
            self.messageBox('Ошибка сохранения', str(e), (("ОК", "ok"),))
        else:
            self.messageBox('Сохранено', 'Конфигурационный файл успешно сохранён', (("ОК", "ok"),))

    def __applyConfig(self):
        try:
            self.config.write()
            gmMain()
        except Exception as e:
            log.error(u"Ошибка: %s" % str(e))
            self.messageBox('Ошибка', str(e), (("ОК", "ok"),))
        else:
            self.messageBox('Выполнено', 'Настройки применены', (("ОК", "ok"),))

    def messageBox(self, title, text, buttons):
        '''
        Функция показа диалогового окна с кнопками
        '''
        log.info("Сообщение: " + text)
        bb = ButtonBar(self.screen, buttons, compact = 1)
        g = GridForm(self.screen, title, 1, 2)
        g.add(Label(text), 0, 0, (0, 0, 0, 1))
        g.add(bb, 0, 1)
        return bb.buttonPressed(g.runOnce())


def listDlg(screen, params, current, title):
    """
    Функция показа диалогового окна со списком
    """
    l = len(params)
    height = l if l < 6 else 6
    scroll = 1 if l > 6 else 0
    li = Listbox(height = height, width = 28, scroll = scroll, returnExit = 1)
    if type(params) == list:
        for param in params:
            li.append(param, param)
    elif type(params) == dict:
        for key in params:
            li.append(params[key], key)

    g = GridForm(screen, title, 1, 1)
    g.add(li, 0, 0)
    g.addHotKey("ESC")

    try:
        li.setCurrent(current)
    except Exception as e:
        pass

    return li.current() if g.runOnce() == li else current

def entryDlg(screen, current, title):
    """
    Функция показа диалогового окна с полем ввода
    """
    entry = Entry(40)
    entry.set(current)
    bb = ButtonBar(screen, (("Применить", "ok"), ("Отмена", "canсel")), compact = 1)
    g = GridForm(screen, title, 1, 2)
    g.add(entry, 0, 0, (0, 0, 0, 1))
    g.add(bb, 0, 1)
    g.addHotKey("ESC")
    g.addHotKey("ENTER")

    result = g.runOnce()
    return entry.value() if bb.buttonPressed(result) == 'ok' or result == "ENTER" else current

def listWithEntryDlg(screen, params, current, title):
    '''
    Основная проблема этого метода заключается в том, что добавляемое вручную значение хардкодно зависит от длины списка
    '''
    if current not in params:
        params.insert(2, current)
    l = len(params)
    height = l if l < 6 else 6
    scroll = 1 if l > 6 else 0
    li = Listbox(height = height, width = 36, scroll = scroll, returnExit = 1)
    for param in params:
        li.append(param, param)

    g = GridForm(screen, title, 1, 1)
    g.add(li, 0, 0)
    g.addHotKey("ESC")
    g.addHotKey("ENTER")

    try:
        li.setCurrent(current)
    except Exception as e:
        pass

    runResult = g.runOnce()

    if runResult == "ENTER":
        if li.current() == manual:
            result = entryDlg(screen, params[2] if len(params) > 3 else '', title).strip()
            if len(result):
                if len(params) == 3:
                    params.insert(2, result)
                else:
                    params[2] = result
                return result
            else:
                return current
        else:
            return li.current()
    else:
        return current

def callBack(obj):
    """
    функция для показа справки по определнному полю в списке настроек
    """
    li = obj[0]
    textBox = obj[1]
    textBox.setText("\n\n\n\n")
    textBox.setText(li.current().specification)

def formatListLine(text, value):
    rawTextLen = len(text.lstrip())
    text = "{:{width}}".format(" " + text + ":", width=30 + rawTextLen/2)
    return text + value

if __name__ == "__main__":
    try:
        m = MainForm()
        m.start()
    except:
        log.error(traceback.format_exc())
    finally:
        m.finish()
