#!/usr/bin/env bash

tmp_conffile=/tmp/xinput_calibrator.conf
dist_codename=`lsb_release -c -s`

export DISPLAY=:0 
export XAUTHORITY=/home/autologon/.Xauthority

rm -f $tmp_conffile

outxrandr=$(xrandr)
primaryresolution=$(sed -n 's/.*primary \([0-9x]*\)+.*/\1/p' <<< $outxrandr)

if [ -z $primaryresolution ]; then
    geometry=""
else
    geometry="--geometry $primaryresolution"
    echo "Разрешение основного экрана: $primaryresolution"
fi

# на trusty просто запускаем калибровку
if [[ "$dist_codename" == "trusty" ]] ; then
    sudo -u autologon xinput_calibrator -v $geometry --output-filename $tmp_conffile
    cp $tmp_conffile /usr/share/X11/xorg.conf.d/99-calibration.conf
    exit 0
fi

function choiceDevice() 
{
    IFS=$'\n'
    local i=1
    local j=1
    for var in $1
    do
        array[ $i ]=$j
        array[ ($i + 1) ]=$var
        devices[$j]=$var
        (( j++ ))
        (( i=($i+2) ))
    done

    local cmd=(dialog --title 'Выберите устройство' --menu 'Выберите устройство для калибровки' 22 76 16)
    local choices=$("${cmd[@]}" "${array[@]}" 2>&1 >/dev/tty)

    echo "${devices[$choices]}"
}

device=$(xinput_calibrator --list)
device=$(sed -n 's/.*Device\s\"\(.*\)\".*/\1/p' <<< $device)
device_count=$(echo -n "$device" | grep -c '^')

case $device_count in
    0 )
        echo "Нет устройств для калибровки"
        exit 1
        ;;
    1 ) ;;
    * )
        device=$(choiceDevice "$device")
        [[ -z $device ]] && exit 1
        ;;
esac

# сбросить старые настройки
xinput set-prop "$device" 'Coordinate Transformation Matrix' 1 0 0 0 1 0 0 0 1

chvt $(cat /tmp/cash_tty)
out=$(xinput_calibrator --geometry "$primaryresolution" --device "$device")

wtot=$(sed -n 's/.*max_x=\([0-9]*\).*/\1/p' <<< $out)
htot=$(sed -n 's/.*max_y=\([0-9]*\).*/\1/p' <<< $out)


minx=$(sed -n 's/.*MinX\"\s\"\([-0-9]*\).*/\1/p' <<< $out)
maxx=$(sed -n 's/.*MaxX\"\s\"\([-0-9]*\).*/\1/p' <<< $out)
miny=$(sed -n 's/.*MinY\"\s\"\([-0-9]*\).*/\1/p' <<< $out)
maxy=$(sed -n 's/.*MaxY\"\s\"\([-0-9]*\).*/\1/p' <<< $out)

wtouch=$(bc <<< "$maxx - $minx")
htouch=$(bc <<< "$maxy - $miny")

c0=$(bc -l <<< "$wtot / $wtouch")
c1=$(bc -l <<< "-1 * $minx / $wtouch")
c2=$(bc -l <<< "$htot / $htouch")
c3=$(bc -l <<< "-1 * $miny / $htouch")

tf_matrix="$c0 0 $c1 0 $c2 $c3 0 0 1"

# применить новые настройки
xinput set-prop "$device" 'Coordinate Transformation Matrix' $tf_matrix

cat >  $tmp_conffile <<EOF
Section "InputClass"
	Identifier "calibration"
	MatchProduct "$device"
	Option "TransformationMatrix" "$tf_matrix"
EndSection
EOF

cp $tmp_conffile /usr/share/X11/xorg.conf.d/99-calibration.conf
